import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { Document } from 'mongoose';
import { Address } from './address.schema';
import { Week } from './week.schema';

export type StructureDocument = Structure & Document;

@Schema({ timestamps: { createdAt: 'createdAt', updatedAt: 'updatedAt' } })
export class Structure {
  @Prop()
  numero: string;

  @Prop()
  createdAt: string;

  @Prop()
  updatedAt: string;

  @Prop()
  structureName: string;

  @Prop()
  structureType: string;

  @Prop()
  description: string;

  @Prop()
  lockdownActivity: string;

  @Prop()
  address: Address;

  @Prop()
  contactPhone: string;

  @Prop()
  contactMail: string;
  @Prop()
  website: string;

  @Prop()
  facebook: string;

  @Prop()
  twitter: string;

  @Prop()
  instagram: string;

  @Prop()
  linkedin: string;

  @Prop()
  pmrAccess: boolean;

  @Prop()
  accessModality: string[];

  @Prop()
  otherDescription: string;

  @Prop()
  labelsQualifications: string[];

  @Prop()
  publics: string[];

  @Prop()
  exceptionalClosures: string;

  @Prop()
  publicsAccompaniment: string[];

  @Prop()
  proceduresAccompaniment: string[];

  @Prop()
  baseSkills: string[];

  @Prop()
  accessRight: string[];

  @Prop()
  socialAndProfessional: string[];

  @Prop()
  parentingHelp: string[];

  @Prop()
  digitalCultureSecurity: string[];

  @Prop()
  equipmentsAndServices: string[];

  @Prop()
  freeWorkShop: boolean;

  @Prop()
  nbComputers: number;

  @Prop()
  nbPrinters: number;

  @Prop()
  nbTablets: number;

  @Prop()
  nbNumericTerminal: number;

  @Prop()
  nbScanners: number;

  @Prop()
  hours: Week;

  @Prop()
  coord: number[];

  @Prop()
  deletedAt: Date;

  @Prop()
  accountVerified: boolean;
}

export const StructureSchema = SchemaFactory.createForClass(Structure);
StructureSchema.index({ '$**': 'text' });
