import { Injectable, CanActivate, ExecutionContext } from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { UserRole } from '../enum/user-role.enum';

@Injectable()
export class RolesGuard implements CanActivate {
  constructor(protected reflector: Reflector) {}

  canActivate(context: ExecutionContext): boolean {
    const roles = this.reflector.get<string[]>('roles', context.getHandler());
    if (!roles) {
      return true;
    }
    const request = context.switchToHttp().getRequest();
    const user = request.user;
    return this.matchRoles(user.role, roles[0]);
  }

  /**
   * Return true if user is admin or if the requested role match the user role.
   * @param userRole user role from request
   * @param expectedRole role requested by the endpoint
   */
  private matchRoles(userRole: number, expectedRole: string): boolean {
    if (userRole === UserRole.admin) {
      return true;
    } else if (userRole === UserRole[expectedRole]) {
      return true;
    } else {
      return false;
    }
  }
}
